const axios = require('axios')
const { configPath, getConfig, setConfig } = require('./config')
const OSS = require('ali-oss')

let aliyunClient = null

async function getAliyunOssClient() {
    if (!aliyunClient) {
        let configJson = getConfig()
        let { aliossUrl, aliossEndpoint, aliossBucketName } = configJson
        if (!aliossUrl) {
            console.log('没有配置阿里云OSS STS服务URL');
            return
        }
        if (!aliossEndpoint) {
            console.log('没有配置阿里云OSS endpoint');
            return
        }
        if (!aliossBucketName) {
            console.log('没有配置阿里云OSS bucketName');
            return
        }
        console.log('授权接口',configJson.aliossUrl);
        let res = await axios.get(configJson.aliossUrl).catch((e) => {
            console.log('获取阿里云OSS授权失败',e);
        })
        if (res) {
            let { data } = res
            aliyunClient = new OSS({
                accessKeyId: data.AccessKeyId,
                accessKeySecret: data.AccessKeySecret,
                stsToken: data.SecurityToken,
                endpoint: aliossEndpoint,
                bucket: aliossBucketName
            });
        }
    }
    return aliyunClient
}

async function uploadFile(file, oss) {
    let client = await getAliyunOssClient()
    if (!client) {
        return 
    }
    const result = await client.put(oss, file)
    return result
}

module.exports = {
    uploadFile
}