
const { createFolder } = require('./utils/fileManage')
const { uploadFile } = require('./aliyunOss')
const fs = require('fs');
const path = require('path');
const JSZIP = require('jszip');
const axios = require('axios')
const qs = require('qs')
const crypto = require('crypto');
const { configPath, getConfig, setConfig } = require('./config')
const { terminal } = require('./terminal')


let versionJsonPath = path.join(process.cwd(), './appBundleVersion.json')

function getVersionJson() {
    let versionJson = {}
    if (fs.existsSync(versionJsonPath)) {
        let buffer = fs.readFileSync(versionJsonPath)
        versionJson = JSON.parse(buffer.toString())
        // console.log('读取versionJson', versionJson);
    }
    return versionJson
}
function setVersionJson(con) {
    let versionJson = getVersionJson()
    versionJson = {
        ...versionJson,
        ...con
    }
    // console.log('设置versionJson', versionJson);
    fs.writeFileSync(versionJsonPath, JSON.stringify(versionJson, null, 4))
}

function setBundleVersionJson(con,path) {
    // console.log('设置versionJson', con);
    fs.writeFileSync(path, JSON.stringify(con, null, 4))
}

async function runBundle(args, config, options) {
    const name = args[0];
    const version = args[1];
    const platform = args[2];
    let { entryFile, dir, description, iosBundleFile, androidBundleFile, git, gitPush, isAliyun } = options

    console.log(`开始 ${name} ${version} ${platform}  ${__dirname} ${process.cwd()} ${entryFile} ${dir} ${iosBundleFile} ${androidBundleFile}  ${git} ${gitPush}`);
    await terminal('rm', ['-rf', dir], {
        cwd: process.cwd(),
    })
    await createFolder(`${dir}`)

    if (platform == 'all' || platform == 'ios') {
        await bundle(name, version, description, dir, 'ios', iosBundleFile, entryFile, isAliyun, git, gitPush)
    }
    if (platform == 'all' || platform == 'android') {
        await bundle(name, version, description, dir, 'android', androidBundleFile, entryFile, isAliyun, git, gitPush)
    }
}

async function bundle(name, version, description, dir, platform, bundleFile, entryFile, isAliyun, git, gitPush) {
    let platform_path = path.resolve(dir, platform)
    await createFolder(platform_path)

    let bundle_file = path.resolve(platform_path, bundleFile)
    let assets_file = `${platform_path}/`

    await terminal(`node`, ['node_modules/react-native/local-cli/cli.js','bundle',
        '--entry-file', entryFile,
        '--bundle-output', bundle_file,
        '--platform', platform,
        '--assets-dest', assets_file,
        '--dev', false], {
        cwd: process.cwd(),
    })

    let versionJson = getVersionJson()
    let platformVersion = versionJson[platform] || {}
    let bundle = platformVersion.bundle || 1
    
    platformVersion = { name, version, bundle, bundleFile: bundleFile };
    versionJson[platform] = platformVersion
    setVersionJson(versionJson)

    let bundleVersionJson_path = path.resolve(dir,platform, 'bundleVersion.json')
    setBundleVersionJson(platformVersion, bundleVersionJson_path)

    // 先生成压缩包
    let platform_zip = path.resolve(dir, `${platform}.zip`)
    await zip(platform_path, platform_zip)

    // 读取压缩包内容计算哈希
    const zipBuffer = fs.readFileSync(platform_zip)
    // const hash = crypto.createHash('sha256')
    const hash = crypto.createHash('md5')
    hash.update(zipBuffer)
    let hex = hash.digest('hex')
    // let hex = hash.digest('base64');

    if (isAliyun) {
        let configJson = getConfig()
        let platformCon = configJson[platform] || {}
        if (hex === platformCon.hash && platformCon.isUpload) {
            console.log('与上次打包一样，请勿重复上传');
            return
        }
        if (bundle <= platformCon.bundle && platformCon.isUpload) {
            console.log('该版本已经上传过，请勿上传相同版本');
            return
        }
        platformCon = { name, version, bundle, bundleFile: bundleFile, hash: hex, isUpload: false }
        let setCon = {}
        setCon[platform] = platformCon
        setConfig(setCon)

        // bundle = bundle + 1

        const result = await uploadFile(platform_zip, `appUpdate/${name}/${version}/${platform}/${hex}.zip`)
        console.log('上传阿里云', result);
        if (!result || result.res.status !== 200) {
            return console.log('阿里云上传失败');
        }
        let url = result.url
        console.log(url);
        if (configJson.pushUpdate) {
            let req = {
                id: platform == 'ios' ? 2 : 1,
                env: 'pro',
                sourceFlag: platform,
                allVersionName: version,
                hotVersion: bundle,
                allVersionInfo: description,
                hotFileUrl:url
            }
            console.log('推送更新参数', req);
            let res2 = await axios.post(configJson.pushUpdate, req).catch((e) => {
                console.log('推送更新失败', e);
            })
            if (res2) {
                if (res2.data.status == 200) {
                    console.log('推送更新', res2.data);
                    platformCon.isUpload = true
                    setConfig(setCon)
                } else {
                    console.log('推送更新失败', res2.data);
                }
            }
        }
    }
}

function hashArr(arr) {
    let str = ''
    for (const hex of arr) {
        str = str + hex.hex
    }
    const hash = crypto.createHash('sha256');
    hash.update(str)
    let hex = hash.digest('hex')
    return hex
}

function hashDir(dirPath) {
    return new Promise((resolve, reject) => {
        fs.readdir(dirPath, async (err, files) => {
            let arr = []
            for (const file of files) {
                let newPath = path.resolve(dirPath, file);
                let flag = fs.statSync(newPath);
                if (flag.isFile()) {
                    const hash = crypto.createHash('sha256');
                    let buffer = fs.readFileSync(newPath)
                    let con = buffer.toString()
                    hash.update(con)
                    let hex = hash.digest('hex')
                    arr.push({ path: file, hex })
                } else if (flag.isDirectory()) {
                    let newArr = await hashDir(newPath);
                    arr = arr.concat(newArr)
                }
            }
            if (err) {
                reject(err)
            } else {
                resolve(arr)
            }
        })
    })
}

function copyDir(prePath, nextPath) {
    return new Promise((resolve, reject) => {
        fs.readdir(prePath, async (err, files) => {
            for (const file of files) {
                let dirPath = path.resolve(prePath, file);
                let nextFile = path.resolve(nextPath, file);
                // console.log(dirPath)
                //检测文件是文件夹还是文件
                let flag = fs.statSync(dirPath);
                //如果为文件
                if (flag.isFile()) {
                    fs.copyFileSync(dirPath, nextFile);
                    //如果为文件夹
                } else if (flag.isDirectory()) {
                    if (!fs.existsSync(nextFile)) {
                        fs.mkdirSync(nextFile);
                    }
                    await copyDir(dirPath, nextFile);
                }
            }
            if (err) {
                reject(err)
            } else {
                resolve()
            }
        })
    })
}

async function zip(path, toPath) {
    const zip = new JSZIP();
    pushZip(zip, path);
    let content = await zip.generateAsync({
        type: 'nodebuffer',
        compression: 'DEFLATE',
        compressionOptions: {
            level: 9
        }
    });
    fs.writeFileSync(toPath, content);
}

function pushZip(zip, pPath) {
    const files = fs.readdirSync(pPath, { withFileTypes: true });
    files.forEach((dirent, index) => {
        let filePath = `${pPath}/${dirent.name}`;
        if (dirent.isDirectory()) {
            let zipFloder = zip.folder(dirent.name);
            pushZip(zipFloder, filePath);
        } else {
            zip.file(dirent.name, fs.readFileSync(filePath));
        }
    });
}

module.exports = {
    runBundle: runBundle
}