
const {runBundle} = require('./bundle')
const {runConfig} = require('./config')
const {runPack} = require('./pack')

module.exports = {
  bundle: {
    command: 'bundle',
    name: 'app-bundle',
    description: 'bundle app , example $ react-native-app-bundle <name> <version> <platform>',
    usage: '<name> <version> <platform> [options]',
    func: runBundle,
    arguments: [
      { name: '<name>', description: 'app name' },
      { name: '<version>', description: 'bundle version' },
      { name: '<platform>', description: 'platform: all ios android' },
    ],
    options: [
      {
        command: '-ef, --entry-file [entryFile]',
        description: '设置js入口文件',
        default: 'index.js',
      },
      {
        command: '-d, --dir [dir]',
        description: '打包出来文件夹名称',
        default: 'appBundle',
      },
      {
        command: '-des, --description [description]',
        description: '更新描述',
      },
      {
        command: '-if, --ios-bundle-file [iosBundleFile]',
        description: 'ios 打出来的js包的名称',
        default: 'index.ios.bundle',
      },
      {
        command: '-af, --android-bundle-file [androidBundleFile]',
        description: 'android 打出来的js包的名称',
        default: 'index.android.bundle',
      },
      {
        command: '-ia, --is-aliyun',
        description: '上传到阿里云',
      },
      {
        command: '-env, --environment',
        description: '环境',
      }
    ]
  },
  pack:{
    command: 'pack',
    name: 'pack android or ios',
    description: 'pack android or ios',
    usage: 'react-native-app-bundle pack <name> <platform> [options]',
    func: runPack,
    arguments: [
      { name: '<name>', description: 'app name' },
      { name: '<platform>', description: 'platform: all ios android' },
    ],
    options: [
      {
        command: '-ip, --is-pack [is]',
        description: '是否编译打包(apk|ipa)',
        default: true,
      },
      {
        command: '-iua, --is-upload-appStore',
        description: 'ios is upload appStore',
      },
      {
        command: '-iuo, --is-upload-oss',
        description: 'is upload pack to alyunOss',
      },
      {
        command: '-iup, --is-upload-pgyer',
        description: 'is upload pack to pgyer',
      },
      {
        command: '-op, --oss-path [path]',
        description: 'pack alyunOss path',
      },
      {
        command: '-apk, --apk-path [path]',
        description: 'apk path'
      },
      {
        command: '-ipa, --ipa-path [path]',
        description: 'ipa path'
      },
      {
        command: '-iu, --is-upload',
        description: 'is upload pack to api',
      },
      {
        command: '-des, --description [description]',
        description: 'description',
      },
    ]
  },
  config: {
    command: 'config',
    name: 'config',
    description: 'config',
    usage: '[options]',
    func: runConfig,
    options: [
      {
        command: '-au, --alioss-url [url]',
        description: 'set aliyun oss sts url',
      },
      {
        command: '-ae, --alioss-endpoint [endpoint]',
        description: 'set aliyun oss endpoint',
      },
      {
        command: '-ab, --alioss-bucketName [bucketName]',
        description: 'set aliyun oss bucketName',
      },
      {
        command: '-pu, --push-update [url]',
        description: 'set push update url',
      }
    ]
  }
};







