
const { terminal, terminalCallback, terminalPassageway } = require('./terminal')
const { uploadFile } = require('./aliyunOss')
const path = require('path');
const fs = require('fs');
const JSZIP = require('jszip');
var convert = require('xml-js');
const axios = require('axios')
const { configPath, getConfig, setConfig } = require('./config')
const inquirer = require('inquirer')
const qs = require('qs')
const FormData = require('form-data')
const qrcode = require('qrcode-terminal');
const { iosExportOptions } = require('./templates');
const { createFolder } = require('./utils/fileManage')

let apk_def_path = '/android/app/build/outputs/apk/release/app-release.apk'
let app_pack = 'appPack'

async function runPack(args, config, options) {
    const name = args[0];
    const platform = args[1];
    let { isPack, isUploadAppStore, ipaPath, apkPath, isUpload, isUploadOss, isUploadPgyer, ossPath, description } = options
    console.log('开始', __dirname, name, platform, options);

    if (isPack === 'false' || isPack === 'f' || isPack === 'no' || isPack === 'n') {
        isPack = false
    }
    let res = {}
    let android_res = null
    let ios_res = null
    // console.log('开始',name,platform,apk,isUpload,isOss,oss,description);
    if (platform == 'all' || platform == 'android') {
        android_res = await packApk(name, isPack, apkPath, isUpload, isUploadOss, isUploadPgyer, ossPath, description)
        if (android_res) {
            res.android = android_res
        } else {
            res.android = { isSuccess: false, err: '打包失败' }
        }
    }
    if (platform == 'all' || platform == 'ios') {
        ios_res = await packIpa(name, isPack, ipaPath, isUploadAppStore, isUpload, isUploadOss, isUploadPgyer, ossPath, description)
        if (ios_res) {
            res.ios = ios_res
        } else {
            res.ios = { isSuccess: false, err: '打包失败' }
        }
    }
    console.log('打包命令执行完毕', res);
    if (android_res && android_res.pgyer_res.isSuccess && android_res.pgyer_res.url) {
        console.log('蒲公英 android 下载地址 ', android_res.pgyer_res.url);
        qrcode.generate(android_res.pgyer_res.url, { small: true });
    }
    if (ios_res && ios_res.pgyer_res.isSuccess && ios_res.pgyer_res.url) {
        console.log('蒲公英 ios 下载地址 ', ios_res.pgyer_res.url);
        qrcode.generate(ios_res.pgyer_res.url, { small: true });
    }
}

async function packIpa(name, isPack, ipaPath, isUploadAppStore, isUpload, isUploadOss, isUploadPgyer, ossPath, description) {
    let configJson = getConfig()
    let ios_pack = path.join(app_pack, 'ios')
    let ios_pack_path = path.resolve(ios_pack)
    let exportPath = path.resolve(ios_pack_path, `${name}`)
    let ipa_xd_path = path.join(ios_pack, name, `${name}.ipa`)
    let ipa_path = ipaPath || path.resolve(ipa_xd_path)
    let aliyun_path = ossPath || configJson.iosPack?.ossPath || `ipa/${name}.ipa`
    let pack_res = { isPack, isSuccess: false }
    let api_res = { isUpload, isSuccess: false }
    let oss_res = { isUploadOss, isSuccess: false }
    let appStore_res = { isUploadAppStore, isSuccess: false }
    let pgyer_res = { isUploadPgyer, isSuccess: false }

    configJson.iosPack = {
        ...configJson.iosPack
    }

    await createFolder(ios_pack_path)

    if (isPack) {
        if (!configJson.iosPack.workspace || !configJson.iosPack.scheme) {
            return console.log('请配置iOS打包参数 iosPack 下的 workspace 和 scheme');
        }
        let archive = getArchivesPath(name)
        try {
            await new Promise(async (resolve, reject) => {
                let proc = terminalCallback((code, res) => {
                    if (code == 0) {
                        resolve(res)
                    } else {
                        reject(code)
                    }
                }, `xcodebuild`, [
                    'archive',
                    '-workspace', configJson.iosPack.workspace,
                    '-scheme', configJson.iosPack.scheme,
                    '-destination', 'generic/platform=iOS',
                    '-archivePath', archive
                ], {
                    cwd: process.cwd(),
                })
                await terminalPassageway(proc, 'xcpretty')
            })

            await terminal('rm', ['-rf', ios_pack_path])
            await createFolder(ios_pack_path)
            let ios_export_file = path.resolve(ios_pack_path, iosExportOptions.name())
            fs.writeFileSync(ios_export_file, iosExportOptions.content({
                method: isUploadAppStore ? iosExportOptions.method.appStore : iosExportOptions.method.adHoc
            }))
            let sh_path = path.join(__dirname, 'xcbuild-safe.sh')

            await terminal('xcrun', [
                sh_path,
                '-exportArchive',
                '-archivePath', archive,
                '-exportPath', exportPath,
                '-exportOptionsPlist', ios_export_file,
            ])

        } catch (error) {
            console.log('打包失败', error);
            return
        }
        pack_res.isSuccess = true
        pack_res.ipa_path = ipa_path
        configJson.iosPack.pack_res = pack_res
        setConfig(configJson)
    }

    console.log('ipa 路径：', ipa_path);
    if (!fs.existsSync(ipa_path)) {
        console.log('ipa 文件不存在');
        return
    }

    let con = fs.readFileSync(ipa_path)
    let zip = await JSZIP.loadAsync(con)
    let bplist = path.resolve(ios_pack_path, 'bplist.plist')
    let xml_plist = path.resolve(ios_pack_path, 'xml_plist.plist')
    let CFBundleShortVersionString = null
    for (const iterator of Object.keys(zip.files)) {
        if (iterator.indexOf('Info.plist') > 0) {
            let arr = iterator.split('/')
            if (arr.length === 3) {
                let file = zip.files[iterator]
                console.log('iterator', file);
                await new Promise((resolve, reject) => {
                    file.async('nodebuffer').then(async (con) => {
                        // console.log('file', con);
                        if (isUploadAppStore) {

                        }
                        fs.writeFileSync(bplist, con)
                        // await terminal('plutil', ['-p', bplist])
                        await terminal('plutil', ['-convert', 'xml1', bplist, '-o', xml_plist])
                        con = fs.readFileSync(xml_plist)
                        console.log('file', con);
                        let res = convert.xml2js(con, { compact: false })
                        let plist = getElementsForKey(res.elements, 'plist')
                        let dict = getElementsForKey(plist.elements, 'dict')
                        let key = ''
                        for (const index in dict.elements) {
                            let elements = dict.elements[index]
                            if (elements.name === 'key') {
                                key = elements.elements[0].text
                            } else {
                                if (key === 'CFBundleShortVersionString') {
                                    CFBundleShortVersionString = elements.elements[0].text
                                    console.log(key, CFBundleShortVersionString);
                                    break
                                }
                            }
                        }
                        resolve()
                    })
                })

                break
            }
        }
    }
    if (!CFBundleShortVersionString) {
        console.log('ipa 解析失败');
    }
    console.log('上次版本', configJson.iosPack?.version);
    if (CFBundleShortVersionString && configJson.iosPack?.version === CFBundleShortVersionString) {
        let res = await inquirer.prompt([{
            type: 'confirm',
            name: 'isContinue',
            message: '版本号和上次的一致 是否继续',
            default: false
        }])
        if (!res.isContinue) {
            return
        }
    }
    configJson.iosPack.version = CFBundleShortVersionString
    setConfig(configJson)

    if (isUploadOss) {
        const result = await uploadFile(ipa_path, aliyun_path)
        console.log('上传阿里云', result);
        if (result.res.status !== 200) {
            return console.log('阿里云上传失败');
        }
        let url = result.url
        configJson.iosPack.isUploadAliyun = true
        configJson.iosPack.ipaUrl = url
        oss_res.isSuccess = true
        oss_res.url = url

        let isPush = false
        if (isUpload) {
            isPush = await pushUpdate('ios', CFBundleShortVersionString, description, url)
        }
        if (isPush) {
            api_res.isSuccess = true
        }
        configJson.iosPack.oss_res = oss_res
        configJson.iosPack.api_res = api_res
        setConfig(configJson)
    }

    if (isUploadPgyer) {
        let data = await pushPgyer('ios', ipa_path)
        if (data) {
            let img_res = await axios({
                url: data.buildQRCodeURL,
                responseEncoding: 'binary'
            })
            let img_path = path.resolve(ios_pack_path, 'pgyer.png')
            fs.writeFileSync(img_path, img_res.data, 'binary')
            pgyer_res.isSuccess = !!data
            pgyer_res.url = data.downloadUrl
            pgyer_res.image = img_path
        }
        configJson.iosPack.pgyer_res = pgyer_res
        setConfig(configJson)
    }

    if (isUploadAppStore) {
        if (!configJson.iosPack?.apiKey || !configJson.iosPack?.apiIssuer) {
            console.log('请配置苹果用户凭证 apiKey apiIssuer');
            return
        }
        try {
            await terminal('xcrun', [
                'altool', '--validate-app',
                '--type', 'ios',
                '-f', ipa_path,
                '--apiKey', configJson.iosPack.apiKey,
                '--apiIssuer', configJson.iosPack.apiIssuer,
            ])
        } catch (error) {
            console.log('ipa校验失败');
            return
        }
        await terminal('xcrun', [
            'altool', '--upload-app',
            '--type', 'ios',
            '-f', ipa_path,
            '--apiKey', configJson.iosPack.apiKey,
            '--apiIssuer', configJson.iosPack.apiIssuer,
        ])
        appStore_res.isSuccess = true
        configJson.iosPack.appStore_res = appStore_res
        setConfig(configJson)
    }
    return {
        pack_res,
        api_res,
        oss_res,
        pgyer_res,
        appStore_res
    }
}

function getElementsForKey(elements, key) {
    for (const iterator of elements) {
        if (iterator.name === key) {
            return iterator
        }
    }
}

function getArchivesPath(name) {
    let date = new Date()
    let year = date.getFullYear().toString();
    let month = addPrefi((date.getMonth() + 1).toString(), 2);
    let day = addPrefi(date.getDate().toString(), 2);
    let hour = addPrefi(date.getHours().toString(), 2);
    let minute = addPrefi(date.getMinutes().toString(), 2);
    let second = addPrefi(date.getSeconds().toString(), 2);
    return `~/Library/Developer/Xcode/Archives/${year}-${month}-${day}/${name} ${year}-${month}-${day}, ${hour}.${minute}.${second}.xcarchive`
}

function addPrefi(nubStr, count) {
    let lent = String(nubStr).length;
    let str = '';
    if (count > lent) {
        for (let i = 0; i < count - lent; i++) {
            str = str + '0'
        }
    }
    return str + nubStr
}

async function packApk(name, isPack, apk, isUpload, isUploadOss, isUploadPgyer, ossPath, description) {

    let configJson = getConfig()
    let android_pack = path.resolve(app_pack, 'android')
    let android_pack_path = path.resolve(android_pack)
    let apk_xd_path = apk || apk_def_path
    let apk_path = path.join(process.cwd(), apk_xd_path)
    let aliyun_path = ossPath || configJson.androidPack?.ossPath || `apk/${name}.apk`

    let pack_res = { isPack, isSuccess: false }
    let api_res = { isUpload, isSuccess: false }
    let oss_res = { isUploadOss, isSuccess: false }
    let pgyer_res = { isUploadPgyer, isSuccess: false }

    configJson.androidPack = {
        ...configJson.androidPack
    }
    await createFolder(android_pack_path)

    try {
        if (isPack) {
            await terminal(`./gradlew`, ['assembleRelease'], {
                cwd: path.join(process.cwd(), './android'),
            })
            pack_res.isSuccess = true
            pack_res.apk_path = apk_path

            let apk_path2 = path.resolve(android_pack_path, `${name}.apk`)
            fs.copyFileSync(apk_path, apk_path2)
            pack_res.apk_copy_path = apk_path2
            configJson.androidPack.pack_res = pack_res
            setConfig(configJson)
        }
    } catch (error) {
        console.log('打包失败', error);
        return
    }

    console.log('apk 路径：', apk_path);
    if (!fs.existsSync(apk_path)) {
        console.log('apk 文件不存在');
        return
    }

    let package = null
    try {
        let arr = await terminal(`aapt`, ['dump', 'badging', apk_path], {
            cwd: process.cwd(),
        })
        let apkInfo = objForStrArr(arr, ':')
        package = objForStrArr(apkInfo.package.split(' '), '=')
        console.log('apk package 信息', package);
    } catch (error) {
        console.log('获取apk信息失败', error);
        return
    }

    let versionName = package.versionName
    if (configJson.androidPack.version === versionName) {
        let res = await inquirer.prompt([{
            type: 'confirm',
            name: 'isContinue',
            message: '版本号和上次的一致 是否继续',
            default: false
        }])
        if (!res.isContinue) {
            return
        }
    }
    console.log('versionName ', versionName);
    configJson.androidPack.version = versionName
    setConfig(configJson)

    if (isUploadOss) {
        const result = await uploadFile(apk_path, aliyun_path)
        console.log('上传阿里云', result);
        if (result.res.status !== 200) {
            return console.log('阿里云上传失败');
        }
        let url = result.url
        oss_res.isSuccess = true
        oss_res.url = url
        let isPush = false
        if (isUpload) {
            isPush = await pushUpdate('android', versionName, description, url)
        }
        api_res.isSuccess = isPush
        configJson.androidPack.api_res = api_res
        configJson.androidPack.oss_res = oss_res
        setConfig(configJson)

        if (configJson.pushUpdate) {
            let req = {
                id: 1,
                env: 'pro',
                sourceFlag: 'android',
                allVersionName: versionName,
                // allVersionInfo: description,
                allFileUrl:url
            }
            console.log('推送更新参数', req);
            let res2 = await axios.post(configJson.pushUpdate, req).catch((e) => {
                console.log('推送更新失败', e);
            })
            if (res2) {
                if (res2.data.status == 200) {
                    console.log('推送更新', res2.data);
                } else {
                    console.log('推送更新失败', res2.data);
                }
            }
        }

    }

    if (isUploadPgyer) {
        // let url = 'https://www.pgyer.com/app/qrcodeHistory/67fded3f4b0c6b75b30aca1975e2536201ecec34f17b83cadb003e95833a53d7'
        let data = await pushPgyer('android', apk_path)
        if (data) {
            let img_res = await axios({
                url: data.buildQRCodeURL,
                responseEncoding: 'binary'
            })
            let img_path = path.resolve(android_pack_path, 'pgyer.png')
            fs.writeFileSync(img_path, img_res.data, 'binary')
            pgyer_res.isSuccess = !!data
            pgyer_res.url = data.downloadUrl
            pgyer_res.image = img_path
        }
        configJson.androidPack.pgyer_res = pgyer_res
        setConfig(configJson)
    }
    return {
        pack_res,
        api_res,
        oss_res,
        pgyer_res
    }
}

async function pushUpdate(platform, version, description, url) {
    let configJson = getConfig()
    if (configJson.pushUpdate) {
        let req = {
            // id: updateInfo.id,
            id: platform == 'ios' ? 2 : 1,
            env: 'pro',
            sourceFlag: platform,
            allVersionName: version,
            allVersionInfo: description
        }
        if (platform === 'ios') {
            req.allIosFileUrl = url
        } else {
            req.allAndroidFileUrl = url
        }
        console.log('推送更新参数', req);
        let res2 = await axios.post(configJson.pushUpdate, req).catch((e) => {
            console.log('推送更新失败', e);
        })
        if (res2) {
            if (res2.data.status == 200) {
                console.log('推送更新', res2.data);
                return true
            } else {
                console.log('推送更新失败', res2.data);
            }
        }
    } else {
        console.log('没有配置上传接口');
    }
}

function objForStrArr(arr, sp) {
    let obj = {}
    arr.forEach((item) => {
        let arr = item.split(sp)
        if (arr && arr.length == 2) {
            obj[arr[0]] = arr[1].replace(/'/g, '')
        }
    })
    return obj
}

async function pushPgyer(buildType, file_path) {
    let configJson = getConfig()
    if (!configJson.pgyerApiKey) {
        console.log('未配置 蒲公英apikey 字段名: pgyerApiKey');
        return false
    }
    let reqData = {
        _api_key: configJson.pgyerApiKey,
        buildType: buildType,
    }
    let res = await axios.post('https://www.pgyer.com/apiv2/app/getCOSToken', qs.stringify(reqData)).catch((e) => {
        console.log('蒲公英 获取上传的 token 失败', e);
    })
    if (!res || res.data?.code !== 0) {
        console.log('蒲公英 获取上传的 token 失败', res);
        return false
    }
    console.log('蒲公英 获取上传的 token 成功', res.data);

    let tokenData = res.data.data
    let reqData2 = {
        key: tokenData.key,
        signature: tokenData.params.signature,
        'x-cos-security-token': tokenData.params['x-cos-security-token'],
        file: fs.createReadStream(file_path)
    }
    let form = new FormData()
    for (let item in reqData2) {
        form.append(item, reqData2[item])
    }
    let length = await new Promise((resolve, reject) => {
        form.getLength((err, length) => {
            if (!err) {
                resolve(length)
            } else {
                reject(err)
            }
        })
    })
    var headers = form.getHeaders()
    headers['content-length'] = length;
    let res2 = await axios.post(tokenData.endpoint, form, {
        maxBodyLength: length,
        headers: headers
    }).catch((e) => {
        console.log('蒲公英 上传应用 失败', e);
    })
    if (!res2 || res2.status !== 204) {
        console.log('蒲公英 上传应用 失败', res2);
        return false
    }
    console.log('蒲公英 上传应用成功', res2.data);
    let data = await getPgyerPush(configJson.pgyerApiKey, tokenData.key)
    if (!data) {
        return false
    }
    return data
}

async function getPgyerPush(api_key, buildKey) {
    let res3 = await axios({
        method: 'get',
        url: 'https://www.pgyer.com/apiv2/app/buildInfo',
        headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
        params: {
            _api_key: api_key,
            buildKey: buildKey
        }
    }).catch((e) => {
        console.log('蒲公英 获取发布应用的信息 失败', e);
    })
    if (!res3) {
        return false
    }
    console.log('蒲公英 获取发布应用的信息', res3.data);
    if (res3.data.code === 1216) {
        console.log('蒲公英 发布应用失败', res3.data.message);
        return false
    }
    if (res3.data.code === 1247) {
        console.log('应用正在发布');
        let res = await new Promise((resolve, reject) => {
            setTimeout(async () => {
                let res = await getPgyerPush(api_key, buildKey)
                resolve(res)
            }, 1000);
        })
        return res
    }
    if (res3.data.code === 0) {
        let data = res3.data.data
        const buildKey = res3.data.data.buildKey;
        let url = `https://www.pgyer.com/${buildKey}`
        data.downloadUrl = url
        return data
    }
}


module.exports = {
    runPack
}