


const { spawn } = require('child_process')
var readline = require('readline');

function terminalPassageway(child_process, ...args) {
    log('spawn', ...args);
    return new Promise((resolve, reject) => {
        const proc = spawn(...args)
        child_process.stdout.unpipe()
        child_process.stdout.pipe(proc.stdin) // 子进程正常流搭到主进程的正常流
        child_process.stderr.pipe(proc.stderr)
        // proc.stdout.pipe(process.stdout) // 子进程正常流搭到主进程的正常流
        proc.stderr.pipe(process.stderr) // 子进程错误流插到主进程的错误流
        let res = ''
        proc.stdout.on('data',(data)=>{
            res = res + data.toString()
            log(data.toString());
        })
        proc.on('close', (code) => {
            log('terminalPassageway close',code);
            process.stdin.unpipe()
            if (code === 0) {
                resolve(res)
            } else {
                reject()
            }
        })
    })
}

function terminalCallback(callback,...args) {
    log('spawn', ...args);
    const proc = spawn(...args) // 在node.js中执行shell一般用spawn，实现从主进程的输出流连通到子进程的输出流
    // proc.stdout.pipe(process.stdout) // 子进程正常流搭到主进程的正常流
    proc.stderr.pipe(process.stderr) // 子进程错误流插到主进程的错误流
    process.stdin.pipe(proc.stdin)

    let res = []
    readline.createInterface({
        input: proc.stdout,
        terminal: false
    }).on('line', function (line) {
        res.push(line)
        log(line);
    });

    proc.on('close', (code) => {
        log('terminalCallback close',code);
        process.stdin.unpipe()
        callback && callback(code,res)
    })
    return proc
}

function terminal(...args) {
    return new Promise((resolve, reject) => {
        terminalCallback((code,res)=>{
            if (code === 0) {
                resolve(res)
            } else {
                reject()
            }
        },...args)
    })
}

function log(...msg) {
    console.log(getTimeStr(),'=> ',...msg);
}

function getTimeStr() {
    let date = new Date()
    let year = date.getFullYear().toString();
    let month = addPrefi((date.getMonth() + 1).toString(), 2);
    let day = addPrefi(date.getDate().toString(), 2);
    let hour = addPrefi(date.getHours().toString(), 2);
    let minute = addPrefi(date.getMinutes().toString(), 2);
    let second = addPrefi(date.getSeconds().toString(), 2);
    return `${year}-${month}-${day} ${hour}:${minute}:${second}`
}

function addPrefi(nubStr, count) {
    let lent = String(nubStr).length;
    let str = '';
    if (count > lent) {
        for (let i = 0; i < count - lent; i++) {
            str = str + '0'
        }
    }
    return str + nubStr
}

module.exports = {
    terminal,
    terminalCallback,
    terminalPassageway
}