import { sync } from 'mkdirp';
import React, { Component } from 'react';
import { View, Text } from 'react-native';
var RNFS = require('react-native-fs');
import { zip, unzip, unzipAssets, subscribe } from 'react-native-zip-archive'
import CryptoJS from 'crypto-js';

const downloadPath = RNFS.DocumentDirectoryPath + '/appBundle.zip';
const unzipPath = RNFS.DocumentDirectoryPath + '/appBundle';
const bundleInfoPath = RNFS.DocumentDirectoryPath + '/bundleInfo.json';
const apkPath = RNFS.DocumentDirectoryPath + '/app.apk';

let appBundleVersion = null
try {
    appBundleVersion = require('../../../appBundleVersion.json')
} catch (error) {

}

export function getDocumentDirectoryPath() {
    return RNFS.DocumentDirectoryPath
}

export function unzipFile(path, toPath) {
    return unzip(path, toPath)
}

export async function deleteDownload() {
    if (await RNFS.exists(downloadPath)) {
        // console.log('删除文件downloadPath');
        await RNFS.unlink(downloadPath).then().catch(e => { console.log('删除失败'); })
    }
    if (await RNFS.exists(unzipPath)) {
        // console.log('删除文件unzipPath');
        await RNFS.unlink(unzipPath).then().catch(e => { console.log('删除失败'); })
    }
}

export async function deleteApk() {
    if (await RNFS.exists(apkPath)) {
        // console.log('删除文件downloadPath');
        await RNFS.unlink(apkPath).then().catch(e => { console.log('删除失败'); })
    }
}

export function downloadApk(url, progress) {
    return downloadFile(url, apkPath, progress)
}

export function downloadBundle(url, progress) {
    return new Promise(async (resolve, reject) => {
        await deleteDownload()
        let error = null
        let resPath = await downloadFile(url, downloadPath, progress)
            .catch((e) => { error = e })
        if (resPath) {
            // 从URL中提取MD5文件名
            const urlParts = url.split('/');
            const expectedMD5 = urlParts[urlParts.length - 1].split('.')[0];
            console.log('下载完成，开始校验文件',expectedMD5);
            try {
                if (await verifyFileMD5(resPath, expectedMD5)) {
                    console.log('MD5校验通过');
                    unzipFile(downloadPath, unzipPath).then((path) => {
                        resolve(path)
                    })
                        .catch(e => reject(e))
                } else {
                    console.log('MD5校验失败');
                    reject('MD5校验失败')
                }
            } catch (error) {
                console.log('文件校验失败', error);
                reject(error)
            }
        } else {
            reject(error)
        }
    })
}

export function getBundleVersion() {
    let bundle = null
    if (appBundleVersion) {
        let con = appBundleVersion[Platform.OS] || {}
        bundle = con.bundle
    }
    return bundle
}

async function verifyFileMD5(filePath, expectedMD5) {
    // 读取文件内容
    const fileContent = await RNFS.readFile(filePath,'base64');
    // 计算文件内容的 MD5 值
    const wordArray = CryptoJS.enc.Base64.parse(fileContent);
    const actualMD5 = CryptoJS.MD5(wordArray).toString();
    if (actualMD5 !== expectedMD5) {
        await RNFS.unlink(filePath);
        console.log(`MD5校验失败，期望值：${expectedMD5}，实际值：${actualMD5}`);
        return false
    }
    return true;
}

export function downloadFile(url, path, progress) {
    return new Promise(async (resolve, reject) => {
        let { jobId, promise } = RNFS.downloadFile({
            fromUrl: url,
            toFile: path,
            begin: ({ jobId, statusCode, contentLength, headers }) => {
                // console.log('开始下载', statusCode, contentLength);
            },
            progress: ({ jobId, contentLength, bytesWritten }) => {
                // console.log('下载进度', bytesWritten / contentLength);
                progress && progress(bytesWritten, contentLength)
            }
        })
        
        promise.then(async (d) => {
            resolve(path)
        }).catch(async (e) => {
            console.log('下载失败', e);
            // 删除可能已下载的部分文件
            if (await RNFS.exists(path)) {
                await RNFS.unlink(path);
            }
            reject(e)
        })
    })
}

export function versionCompare(version, newVersion) {
    let arr = version.split('.')
    let arr2 = newVersion.split('.')
    let length = Math.min(arr.length, arr2.length)
    for (let i = 0; i < length; i++) {
        if (arr[i] < arr2[i]) {
            return true
        } else if (arr[i] > arr2[i]) {
            return false
        }
    }
    if (arr.length < arr2.length) {
        return true
    }
    return false
}
